#!/usr/bin/env python
"""
This simple example is used for the line-by-line tutorial
that comes with pygame. It is based on a 'popular' web banner.
Note there are comments here, but for the full explanation, 
follow along in the tutorial.
"""


#Import Modules
import os, pygame, operator, math
from copy import copy
from pygame.locals import *

if not pygame.font: print 'Warning, fonts disabled'
if not pygame.mixer: print 'Warning, sound disabled'

from scrollGroup import ScrollSpriteGroup, IdlerSpriteGroup


def allKUPs(x): return x.type == KEYUP

controls = {
	'key_jump': K_UP,
}



#-----------------------------------------------------------------------------
def load_png(name):
    fullname = os.path.join('data', name)
    try:
        image = pygame.image.load(fullname)
	if image.get_alpha is None:
		print "no alpha"
		image = image.convert()
	else:
		image = image.convert_alpha()
    except pygame.error, message:
        print 'Cannot load image:', fullname
        raise SystemExit, message
    return image, image.get_rect()

#-----------------------------------------------------------------------------
def load_sound(name):
    class NoneSound:
        def play(self): pass
    if not pygame.mixer or not pygame.mixer.get_init():
        return NoneSound()
    fullname = os.path.join('data', name)
    try:
        sound = pygame.mixer.Sound(fullname)
    except pygame.error, message:
        print 'Cannot load sound:', fullname
        raise SystemExit, message
    return sound


#------------------------------------------------------------------------------
class Explosion(pygame.sprite.Sprite):
	def __init__(self, position):
		pygame.sprite.Sprite.__init__(self) #call Sprite initializer

		self.image_1, self.rect_1 = load_png('explode_text.png')

		self.image_2, self.rect_2 = load_png('explode_star1.png')

		self.image_3, self.rect_3 = load_png('explode_star2.png')

		self.rect = self.rect_1.unionall( (self.rect_1,
		                                  self.rect_2,
		                                  self.rect_3) ).inflate(30,30)
		self.rect.topleft = [0,0]
		self.rect_1.centerx = self.rect.centerx
		self.rect_1.bottom = self.rect.bottom
		self.rect_2.center = self.rect.center
		self.rect_3.center = self.rect.center
		self.rect.center = position
		self.displayRect = self.rect.move(0,0)

		self.image = pygame.Surface( self.rect.size )
		self.counter = 0
	#---------------------------------------------------------------------
	def NotifyDirtyScreen(self, bgMangr ):
		pass
	#----------------------------------------------------------------------
	def update(self):
		self.counter += 4

		if self.counter >= 254:
			self.kill()
			return

		#--------1
		self.rect_1.move_ip( 0, -1 )
		img = self.image_1.convert()
		img.set_colorkey( img.get_at( [0,0] ) )
		img.set_alpha( 255 - (self.counter +2) )
		img = img.convert_alpha()

		#--------2
		self.rect_2.move_ip( -1, 0 )
		img2= self.image_2.convert()
		img2.set_colorkey( img2.get_at( [0,0] ) )
		img2.set_alpha( 255 - self.counter )
		img2 = img2.convert_alpha()

		#--------2
		self.rect_3.move_ip( 1, 0 )
		img3= self.image_3.convert()
		img3.set_colorkey( img3.get_at( [0,0] ) )
		img3.set_alpha( 15 + self.counter )
		img3 = img3.convert_alpha()

		self.image = pygame.Surface( self.rect.size ).convert_alpha()
		self.image.fill( [0,0,0,0] )
		self.image.blit( img2, self.rect_2.topleft )
		self.image.blit( img3, self.rect_3.topleft )
		self.image.blit( img, self.rect_1.topleft )

#------------------------------------------------------------------------------
class EffectManager:
	#----------------------------------------------------------------------
	def __init__(self, spriteGroup):
		self.spriteGroup = spriteGroup
	
	#----------------------------------------------------------------------
	def PlayEffect(self, effectName, position):
		explosion = Explosion( position )
		self.spriteGroup.add( explosion )

#-----------------------------------------------------------------------------
class DumbArtificialIntelligence:
	def __init__( self, sprite ):
		self.sprite = sprite

	#---------------------------------------------------------------------
	def DecideMove(self):
		"""Returns the change that it wants to make in the position"""
		s = self.sprite
		change = map( operator.mul, s.moveState, 
		              s.speed )
		change = map( int, map( math.ceil, change ) )

		if change == [0,0]:
			return

		#oldPos = self.rect.move( (0,0) )
		#self.rect.move_ip( change )
		newPos = s.rect.move( change )

		global Solids
		limiter = s.Blocked( newPos, Solids )
		if limiter.topSprite is not None:
			s.BumpTop( limiter.topSprite )
			limiter.topSprite.BumpBottom( s )
		if limiter.right is not None:
			s.SetVelocityLeft()
			limiter.rightSprite.BumpLeft( s )
		if limiter.bottomSprite is not None:
			s.BumpBottom( limiter.bottomSprite )
			limiter.bottomSprite.BumpTop( s )
			if s.moveState[0] == 0:
				s.SetVelocityLeft()
		else:
			#nothing below - falling in the air
			if s.moveState[0] != 0:
				s.moveState[0] = 0
			pass
		if limiter.left is not None:
			limiter.leftSprite.BumpRight( s )
			s.SetVelocityRight()

		s.rect = newPos.move( 0,0 )


GRAVITY = 0.09
FRICTION = 0.6
MAX_FALL = 3

#-----------------------------------------------------------------------------
def ApplyFriction( obj ):
	friction = FRICTION

	if hasattr( obj, 'frictionCoeff' ):
		friction *= obj.frictionCoeff

	return obj.moveState[0] - (obj.moveState[0] * friction)

#-----------------------------------------------------------------------------
class Fallable:
	"""interface for things that can fall"""
	#----------------------------------------------------------------------
	def Fall( self ):
		maxFall = MAX_FALL
		fallRate = GRAVITY
		if hasattr( self, 'fallLimit' ):
			maxFall = self.fallLimit
		if hasattr( self, 'featherFall' ):
			fallRate = GRAVITY - self.featherFall


		if self.moveState[1] < maxFall:
			self.moveState[1] = self.moveState[1] + fallRate

#------------------------------------------------------------------------------
class Movable:
	"""interface for things that can fall"""
	#----------------------------------------------------------------------
	def SetVelocityLeft( self ):
		self.moveState[0] = -1

	#----------------------------------------------------------------------
	def SetVelocityRight( self ):
		self.moveState[0] = 1

	#----------------------------------------------------------------------
	def StopLeft( self ):
		if self.moveState[0] is -1:
			self.moveState[0] = 0
	
	#----------------------------------------------------------------------
	def StopRight( self ):
		if self.moveState[0] is 1:
			self.moveState[0] = 0

	#----------------------------------------------------------------------
	def BumpLeft(self, bumpSprite ):
		pass
	#----------------------------------------------------------------------
	def BumpRight(self, bumpSprite ):
		pass
	#----------------------------------------------------------------------
	def BumpBottom(self, bottomSprite ):
		if self.moveState[1] > 0:
			self.moveState[1] = 0
	#----------------------------------------------------------------------
	def BumpTop(self, topSprite ):
		self.jumping = 0
		self.falling = 1
		if self.moveState[1] < 0:
			self.moveState[1] = 0


	#----------------------------------------------------------------------
	def Blocked( self, newRect, spriteGroup ):
		"""See if self would be blocked by sprites in spriteGroup
		if self were to move to newRect (a Rect)
		Tries to change newRect to accommodate the blockers"""
		limiter = BoundingBox()
		oldPos = self.rect.move( 0,0 )

		self.rect = newRect.move( 0,0 )

		colliders = pygame.sprite.spritecollide( self, spriteGroup, 0)

		self.rect = oldPos


		if len(colliders)  is not 0:
			for sprite in colliders:

				if ( limiter.top is None \
				   or sprite.rect.bottom > limiter.top ) \
				   and newRect.top      < sprite.rect.bottom \
				   and oldPos.top        >= sprite.rect.bottom:
					newRect.top         = sprite.rect.bottom
					limiter.top        = sprite.rect.bottom
					limiter.topSprite  = sprite

				if ( limiter.right is None  \
				   or sprite.rect.left  < limiter.right ) \
				   and newRect.right    > sprite.rect.left \
				   and oldPos.right      <= sprite.rect.left:
					newRect.right         = sprite.rect.left
					limiter.right        = sprite.rect.left
					limiter.rightSprite  = sprite

				if ( limiter.bottom is None \
				   or sprite.rect.top < limiter.bottom ) \
				   and newRect.bottom > sprite.rect.top \
				   and oldPos.bottom   <= sprite.rect.top:
					newRect.bottom        = sprite.rect.top
					limiter.bottom       = sprite.rect.top
					#TODO: why does this work with a setter?
					limiter.SetBottom( sprite )

				if ( limiter.left is None \
				   or sprite.rect.right  > limiter.left ) \
				   and newRect.left   < sprite.rect.right \
				   and oldPos.left     >= sprite.rect.right:
					newRect.left       = sprite.rect.right
					limiter.left       = sprite.rect.right
					limiter.leftSprite = sprite
			

		return limiter

#------------------------------------------------------------------------------
class Bouncable(Movable):
	#----------------------------------------------------------------------
	def BumpBottom(self, bottomSprite ):
		if self.moveState[1] > 0:
			self.moveState[1] = -3
	#def Blocked( self, newpos, group ):
		#limiter = Movable.Blocked(self, newpos, group )
		#if limiter.leftSprite: print limiter
		#return limiter

#-----------------------------------------------------------------------------
class SimpleSprite(pygame.sprite.Sprite):
	def __init__(self, imgName=None ):
		pygame.sprite.Sprite.__init__(self) #call Sprite initializer
		if imgName:
			self.image, self.rect = load_png(imgName)
		else:
			self.image = pygame.Surface( (100,100) )
			self.image.fill( (200,0,0) )
			self.rect = self.image.get_rect()
		self.displayRect = self.rect.move(0,0)
	#---------------------------------------------------------------------
	def NotifyDirtyScreen(self, bgMangr ):
		pass
	#---------------------------------------------------------------------
	def NotifyOutOfBounds(self, bounds ):
		print "WENT OFF THE MAP"
	#---------------------------------------------------------------------
	def update(self):
		pass
	#----------------------------------------------------------------------
	def BumpLeft(self, bumpSprite ):
		pass
	#----------------------------------------------------------------------
	def BumpRight(self, bumpSprite ):
		pass
	#----------------------------------------------------------------------
	def BumpTop(self, bumpSprite ):
		pass
	#----------------------------------------------------------------------
	def BumpBottom(self, bumpSprite ):
		pass
	#---------------------------------------------------------------------
	def SetImg(self, newSurface, newRect=None):
		self.image = newSurface
		if newRect:
			self.rect  = newRect
		else:
			self.rect  = newSurface.get_rect()

#-----------------------------------------------------------------------------
class CharactorHUD(SimpleSprite):
	def __init__(self):
		SimpleSprite.__init__( self, 'hud.png' )
		self.originalImg = self.image.convert_alpha()
		self.ChangeHealth(3)


	def ChangeHealth( self, amount ):
		copyImg = self.originalImg.convert_alpha()
		red = (255,0,0)
		for i in range(0,amount):
			pos = 18 + i*28
			pygame.draw.circle( copyImg, red, (pos, 56), 10)

		self.image = copyImg
		return
		


#classes for our game objects
#-----------------------------------------------------------------------------
class GameLevel:
	def __init__(self, bgMangr ):
		self.bgMangr = bgMangr

		self.charGroup    = ScrollSpriteGroup(self.bgMangr)
		self.enemyGroup   = ScrollSpriteGroup(self.bgMangr)
		self.neutralGroup = ScrollSpriteGroup(self.bgMangr)
		self.effectGroup  = ScrollSpriteGroup(self.bgMangr)
		self.idlers       = IdlerSpriteGroup(self.bgMangr)
		self.hudGroup     = pygame.sprite.RenderUpdates()

		self.effects = EffectManager( self.effectGroup )

		self.resurrect = 1

		crate = Crate(self, (200,100))
		crate.SetLevel( self )
		Solids.add( crate )
		self.idlers.add( crate )

		crate = BouncingCrate(self, (600,100))
		crate.SetLevel( self )
		Solids.add( crate )
		self.idlers.add( crate )


		chimp = Chimp(self, (900,50))
		chimp.SetLevel( self )
		self.idlers.add( chimp )
		Solids.add( chimp )
		chimp = Chimp(self, (1800,200))
		chimp.SetLevel( self )
		self.idlers.add( chimp )
		Solids.add( chimp )

		hudInfo = CharactorHUD( )
		self.hudGroup.add( hudInfo )

		self.charactor = Charactor(self.bgMangr)
		self.charactor.SetLevel( self )
		self.charactor.SetHealthListener( hudInfo )
		self.charGroup.add( self.charactor )
		Solids.add( self.charactor )


		self.listenKeys = {}

		plat = SimpleSprite('plat1.png')
		plat.rect.move_ip( 100, 500 )
		plat.rect.inflate_ip( -12, -6 )
		Solids.add( plat )
		self.neutralGroup.add( plat )
		plat = SimpleSprite('plat2.png')
		plat.rect.move_ip( 400, 200 )
		plat.rect.inflate_ip( -12, -6 )
		Solids.add( plat )
		self.neutralGroup.add( plat )
		plat = SimpleSprite('plat1.png')
		plat.rect.move_ip( 700, 400 )
		plat.rect.inflate_ip( -12, -6 )
		Solids.add( plat )
		self.neutralGroup.add( plat )
		plat = SimpleSprite('plat1.png')
		plat.rect.move_ip( 1400, 680 )
		plat.rect.inflate_ip( -12, -6 )
		Solids.add( plat )
		self.neutralGroup.add( plat )
		plat = SimpleSprite('plat1.png')
		plat.rect.move_ip( 2400, 680 )
		plat.rect.inflate_ip( -12, -6 )
		Solids.add( plat )
		self.neutralGroup.add( plat )
		plat = SimpleSprite('plat2.png')
		plat.rect.move_ip( 3000, 580 )
		plat.rect.inflate_ip( -12, -6 )
		Solids.add( plat )
		self.neutralGroup.add( plat )
		plat = SimpleSprite('plat2.png')
		plat.rect.move_ip( 3400, 380 )
		plat.rect.inflate_ip( -12, -6 )
		Solids.add( plat )
		self.neutralGroup.add( plat )
		plat = SimpleSprite('plat1.png')
		plat.rect.move_ip( 4000, 580 )
		plat.rect.inflate_ip( -12, -6 )
		Solids.add( plat )
		self.neutralGroup.add( plat )

	#---------------------------------------------------------------------
	def SpriteBirth(self, sprite):
		self.idlers.remove( sprite )
		if isinstance( sprite, Chimp ):
			self.enemyGroup.add( sprite )
		else:
			self.neutralGroup.add( sprite )

		Solids.add( sprite )

	#---------------------------------------------------------------------
	def SpriteDeath(self, sprite):
		if self.resurrect:
			sprite.kill()
			sprite.Resurrect()
			self.idlers.add( sprite )
		else:
			sprite.kill()


	
	#---------------------------------------------------------------------
	def SignalKey( self, event, remainingEvents ):
		if self.listenKeys.has_key( event.key ):
			self.listenKeys[event.key]()
		else:
			self.charactor.SignalKey( event )
		
	#---------------------------------------------------------------------
	def DoGraphics( self, screen, display ):
		self.charGroup.clear( screen )
		self.enemyGroup.clear( screen )
		self.idlers.clear( screen )
		#Solids.clear(screen)
		self.neutralGroup.clear(screen)
		self.effectGroup.clear(screen)
		self.hudGroup.clear( screen, self.bgMangr.GetBgSurface )

		self.charGroup.update()
		self.enemyGroup.update()
		self.neutralGroup.update( )
		self.effectGroup.update( )
		self.idlers.update()
		#Solids.update()
		self.hudGroup.update()

		changedRects =  self.neutralGroup.draw(screen)
		changedRects += self.effectGroup.draw(screen)
		changedRects += self.charGroup.draw(screen)
		changedRects += self.enemyGroup.draw(screen)
		changedRects += self.hudGroup.draw(screen)
		display.update( changedRects )

		
#-----------------------------------------------------------------------------
class BoundingBox:
	def __init__(self):
		self.top = None
		self.right = None
		self.bottom = None
		self.left = None

		self.topSprite = None;
		self.rightSprite = None;
		self.bottomSprite = None;
		self.leftSprite = None;

	#---------------------------------------------------------------------
	def SetBottom(self, sprite ):
		self.bottomSprite = sprite

#-----------------------------------------------------------------------------
class Charactor(pygame.sprite.Sprite, Fallable, Movable):
    """moves a charactor on the screen, following the mouse"""
    def __init__(self, bgMangr):
        pygame.sprite.Sprite.__init__(self) #call Sprite initializer
	self.bgMangr = bgMangr
	self.level = None

        self.image, self.rect = load_png('charactor.png')
	self.rightFacingImg = self.image
	self.leftFacingImg = pygame.transform.flip( self.image, 1, 0)

	self.rect.midbottom = ( 300, 200 )
	self.displayRect = self.rect.move(0,0)
        #self.area = screen.get_rect().inflate(50,50)
	#self.oldPos = self.rect

	self.listenKeys = { K_UP: self.MoveUp,
	                    K_DOWN: self.MoveDown,
	                    K_LEFT: self.SetVelocityLeft,
	                    K_RIGHT: self.SetVelocityRight,
	                    K_SPACE: self.MoveUp
	                  }

	self.offKeys = { K_UP: self.StopJump,
	                    K_DOWN: self.StopDown,
	                    K_LEFT: self.StopLeft,
	                    K_RIGHT: self.StopRight,
	                    K_SPACE: self.StopJump
	                  }

	self.moveState = [0,0]
	self.speed     = [10,10]
	self.jumping = 0
	self.ducking = 0
	self.falling = 1

	self.health  = 3
	self.invulnerable = 0

	self.snd_jump = load_sound('jump.wav')
	self.snd_land = load_sound('land.wav')

    #--------------------------------------------------------------------------
    def SetLevel(self, level):
    	self.level = level

    #--------------------------------------------------------------------------
    def SetHealthListener(self, listener):
    	self.healthListener = listener

    #--------------------------------------------------------------------------
    def NotifyDirtyScreen(self, bgMangr):
		pass
    #--------------------------------------------------------------------------
    def NotifyOutOfBounds(self, bounds):
		if 'bottom' in bounds:    	
			print "WENT OFF THE MAP"
			raise SystemExit()

    #--------------------------------------------------------------------------
    def update(self):
	if self.jumping:
		self.ContinueJumping()
	elif self.falling:
		self.Fall()

	self.Move()

	if self.moveState[0] < 0:
		self.image = self.rightFacingImg
	if self.moveState[0] > 0:
		self.image = self.leftFacingImg

	self.bgMangr.NotifyPlayerSpritePos(self.rect)

	self.falling = 1


    #--------------------------------------------------------------------------
    def Move( self ):
	#the change in position is the direction they're moving times the
	# speed they're going
    	change = map(operator.mul, self.moveState, self.speed )

	if change == [0,0]:
		return

        newPos = self.rect.move( change )

	global Solids

	limiter = self.Blocked( newPos, Solids )
	if limiter.topSprite is not None:
		self.BumpTop( limiter.topSprite )
	if limiter.right is not None:
		self.BumpRight( limiter.rightSprite )
		limiter.rightSprite.BumpLeft( self )
	if limiter.bottomSprite is not None:
		self.BumpBottom( limiter.bottomSprite )
		limiter.bottomSprite.BumpTop( self )
	if limiter.left is not None:
		self.BumpLeft( limiter.leftSprite )
		limiter.leftSprite.BumpRight( self )


	self.rect = newPos.move( 0,0 )


	#----------------------------------------------------------------------
    def BumpLeft(self, bumpSprite ):
		enemyGroup = self.level.enemyGroup
		if bumpSprite in enemyGroup.sprites():
			self.EnemyBump( bumpSprite )

	#----------------------------------------------------------------------
    def BumpRight(self, bumpSprite ):
		enemyGroup = self.level.enemyGroup
		if bumpSprite in enemyGroup.sprites():
			self.EnemyBump( bumpSprite )

    #--------------------------------------------------------------------------
    def BumpBottom(self, bottomSprite ):
    	if self.moveState[1] > 2:
		self.snd_land.play()
	#self.falling = 0
	self.moveState[1] = 0

    #--------------------------------------------------------------------------
    def EnemyBump(self, bumpSprite ):
    	if self.invulnerable:
		return
	self.health -= 1
	self.healthListener.ChangeHealth( self.health )
	
    #--------------------------------------------------------------------------
    def MoveUp( self ):
	if not self.jumping:
		self.Jump()

    #--------------------------------------------------------------------------
    def Jump( self ):
    	JUMP_POWER = 2.1
	self.snd_jump.play()
	self.jumping = 1
	self.falling = 0
	self.moveState[1] = self.moveState[1] -JUMP_POWER

    #--------------------------------------------------------------------------
    def ContinueJumping( self ):
	self.moveState[1] = self.moveState[1] + GRAVITY
    	if self.moveState[1] >= 0:
		self.jumping = 0
		self.falling = 1
		return
    #--------------------------------------------------------------------------
    def StopJump( self ):
    	if self.jumping:
		self.moveState[1] = 0
		self.jumping = 0
		self.falling = 1
	#if self.moveState[1] is -1:
		#self.moveState[1] = 0




    #--------------------------------------------------------------------------
    def MoveDown( self ):
	if not self.jumping:
		self.ducking = 1

    
    #--------------------------------------------------------------------------
    def StopDown( self ):
	self.ducking = 0
	if self.moveState[1] is 1:
		self.moveState[1] = 0
    
    #--------------------------------------------------------------------------
    def SignalKey( self, event ):
		if self.listenKeys.has_key( event.key ) \
		  and event.type is KEYDOWN:
			self.listenKeys[event.key]( )
		elif self.offKeys.has_key( event.key ) \
		  and event.type is KEYUP:
			self.offKeys[event.key]( )



#-----------------------------------------------------------------------------
class Crate(pygame.sprite.Sprite, Fallable, Movable):
	def __init__(self, level, origin=(100,100)):
		pygame.sprite.Sprite.__init__(self) #call Sprite intializer
		self.image, self.rect = load_png('crate.png')
		self.origin = origin
		self.rect.topleft = self.origin
		self.displayRect = self.rect.move( 0,0 )

		self.active = 0
		self.moveState = [0,0]
		self.speed     = [3,3]
		self.falling = 1
		self.frictionCoeff = 1.0

	#----------------------------------------------------------------------
	def update(self):
		if self.falling:
			self.Fall()

		change = map( operator.mul, self.moveState, self.speed )
		#change = map( int, map( math.ceil, change ) )

		if change == [0,0]:
			return

		newPos = self.rect.move( change )

		global Solids
		limiter = self.Blocked( newPos, Solids )
		if limiter.topSprite is not None:
			self.BumpTop( limiter.topSprite )
			limiter.topSprite.BumpBottom( self )
		if limiter.right is not None:
			self.BumpRight( limiter.rightSprite )
			limiter.rightSprite.BumpLeft( self )
		if limiter.bottomSprite is not None:
			self.BumpBottom( limiter.bottomSprite )
			limiter.bottomSprite.BumpTop( self )
			#apply friction
			self.moveState[0] = ApplyFriction(self)
		if limiter.left is not None:
			self.BumpLeft( limiter.leftSprite )
			limiter.leftSprite.BumpRight( self )

		self.rect = newPos.move( 0,0 )

		self.falling = 1

	#----------------------------------------------------------------------
	def BumpLeft(self, bumpSprite):
		self.moveState[0] = 1
	#----------------------------------------------------------------------
	def BumpRight(self, bumpSprite):
		self.moveState[0] = -1

	#----------------------------------------------------------------------
	def NotifyDirtyScreen(self, bgMangr):
		if not self.active and bgMangr.SpriteIsVisible( self.rect ):
			self.active = 1
			self.level.SpriteBirth( self )

	#----------------------------------------------------------------------
	def NotifyOutOfBounds(self, bounds):
		print "WENT OFF THE MAP"
		self.Die()

	#----------------------------------------------------------------------
	def SetLevel(self, level):
		self.level = level

	#----------------------------------------------------------------------
	def Die(self):
		self.active = 0
		self.level.SpriteDeath(self)
	#----------------------------------------------------------------------
	def Resurrect(self):
		self.rect.topleft = self.origin

#-----------------------------------------------------------------------------
class BouncingCrate(Crate, Fallable, Bouncable):
	def __init__(self, level, origin=(100,100)):
		Crate.__init__( self, level, origin )
		self.frictionCoeff = 0.2

#-----------------------------------------------------------------------------
class Chimp(pygame.sprite.Sprite, Fallable, Movable):
	"""moves a monkey critter across the screen. it can spin the
	   monkey when it is punched."""
	def __init__(self, level, origin=(100,100)):
		pygame.sprite.Sprite.__init__(self) #call Sprite intializer
		self.image, self.rect = load_png('chimp.png')
		self.leftFacingImg = self.image
		self.rightFacingImg = pygame.transform.flip( self.image, 1, 0)
		self.origin = origin
		self.rect.topleft = self.origin
		self.displayRect = self.rect.move( 0,0 )

		self.active = 0
		self.moveState = [0,1]
		self.speed     = [6,6]
		self.falling = 1
		self.dizzy = 0
		self.dying = 0
		self.health = 100
		self.level = level

		self.brain = DumbArtificialIntelligence( self )
		self.SetVelocityRight()

		self.snd_htop = load_sound('top.wav')

	#----------------------------------------------------------------------
	def NotifyDirtyScreen(self, bgMangr):
		if not self.active and bgMangr.SpriteIsVisible( self.rect ):
			self.active = 1
			self.level.SpriteBirth( self )
	#----------------------------------------------------------------------
	def NotifyOutOfBounds(self, bounds):
		if not self.dying:
			print "WENT OFF THE MAP"
			self.Die()

	#----------------------------------------------------------------------
	def SetLevel(self, level):
		self.level = level

	#----------------------------------------------------------------------
	def update(self):
		"walk or spin, depending on the monkeys state"
		if self.dizzy:
			self._spin()
		elif self.dying:
			self.Dying()
		else:

			if self.falling:
				self.Fall()

			change = self.brain.DecideMove()
			#self._walk()

			if self.moveState[0] < 0:
				self.image = self.rightFacingImg
			if self.moveState[0] > 0:
				self.image = self.leftFacingImg

			self.falling = 1


	#----------------------------------------------------------------------
	def Dying(self):
		self.level.SpriteDeath(self)
		return
		
		self.dying = self.dying + 12
		if self.dying >= 360:
			self.dying = 0
			self.image = self.rightFacingImg
			self.level.SpriteDeath(self)

	#----------------------------------------------------------------------
	def _spin(self):
		"spin the monkey image"
		center = self.rect.center
		self.dizzy = self.dizzy + 12
		if self.dizzy >= 360 or self.dying:
			self.dizzy = 0
			self.image = self.rightFacingImg
		else:
			rotate = pygame.transform.rotate
			self.image = rotate( self.rightFacingImg, 
			                     self.dizzy )
		self.rect = self.image.get_rect()
		self.rect.center = center

	#----------------------------------------------------------------------
	def Die(self):
		self.level.effects.PlayEffect( 'explosion', 
		                               self.rect.center )
		self.active = 0
		self.dying  = 1

	#----------------------------------------------------------------------
	def Resurrect(self):
		self.dying  = 0
		self.health = 100
		self.rect.topleft = self.origin

	#----------------------------------------------------------------------
	def BumpTop(self, bumpSprite ):
		damage = 60
		"this will cause the monkey to start spinning"
		if not self.dizzy and not self.dying:
			self.snd_htop.play()
			self.dizzy = 1
			self.health = self.health - damage
			if self.health < 0:
				self.health = 0
				self.Die()



RESOLUTION = (640,480)
vectorSum = lambda a,b: (a[0]+b[0], a[1]+b[1])

#-----------------------------------------------------------------------------
class BackgroundManager:
	def __init__(self, screen):
		self.screen = screen
		self.image, self.rect = load_png('backgr.png')
		#garbage, self.rect = load_png('backgr.png')
		#self.image = pygame.Surface( self.rect.size )
		self.offset = [0,0]
		self.dirty = 0
		#self.image.set_clip( self.offset[0], self.offset[1],
		                     #RESOLUTION[0], RESOLUTION[1] )
	def BlitSelf( self, surface ):
		"""This is called when self.dirty is true and surface is usually
		the main pygame display surface"""
		srcRect = Rect( self.offset[0], self.offset[1],
	                     RESOLUTION[0], RESOLUTION[1] )
		surface.blit( self.image, (0,0), srcRect )

	def GetBgSurface(self, drawToSurface, clearRect):
		offsetRect = clearRect.move(0,0)
		offsetRect.topleft = vectorSum(offsetRect.topleft, self.offset)
		drawToSurface.blit( self.image, clearRect, offsetRect )

	def GetBackground(self):
		return self.image

	def OutOfBounds( self, physRect ):
		result = []
		if physRect.x > self.rect.right:
			result.append( 'right' )
		if physRect.y > self.rect.bottom:
			result.append( 'bottom' )
		if physRect.right < self.rect.left:
		  	result.append( 'left' )
		if physRect.bottom < self.rect.top:
		  	result.append( 'top' )

		return result

	def GetDisplayCenter( self, physRect ):
		return (physRect.centerx - self.offset[0], 
		        physRect.centery - self.offset[1] )

	def SpriteIsVisible( self, physRect ):
		screenRect = self.screen.get_rect()
		viewableRect = screenRect.move( self.offset )
		return viewableRect.colliderect( physRect )

	def NotifyPlayerSpritePos( self, physRect ):
		screenRect = self.screen.get_rect()
		self.dirty = 0
		screenLocationX = physRect.x - self.offset[0]
		screenLocationY = physRect.y - self.offset[1]
		x1Bound = screenRect.centerx - 100
		x2Bound = screenRect.centerx + 100
		y1Bound = screenRect.centery - 100
		y2Bound = screenRect.centery + 100
		if screenLocationX > x2Bound \
		  and self.offset[0] != self.rect.width:
			self.offset[0] = min( self.rect.width -RESOLUTION[0],
			                      physRect.x - x2Bound )
			self.dirty = 1
		elif screenLocationX < x1Bound \
		  and self.offset[0] != self.rect.x:
			self.offset[0] = max( self.rect.x,
			                      physRect.x - x1Bound )
			self.dirty = 1

		if screenLocationY > y2Bound \
		  and self.offset[1] != self.rect.height:
			self.offset[1] = min( self.rect.height -RESOLUTION[1],
			                      physRect.y - y2Bound )
			self.dirty = 1
		elif screenLocationY < y1Bound \
		  and self.offset[1] != self.rect.y:
		  	##print physRect
		  	#print screenLocationY, "vs", y1Bound
			self.offset[1] = max( self.rect.y,
			                      physRect.y - y1Bound )
			self.dirty = 1


		return self.GetDisplayCenter( physRect )


	
#-----------------------------------------------------------------------------
def main():
	"""this function is called when the program starts.
	   it initializes everything it needs, then runs in
	   a loop until the function returns."""
#Initialize Everything
	global screen
	screen = None
	pygame.init()
	screen = pygame.display.set_mode(RESOLUTION)
	pygame.display.set_caption('Scroll, Jump and Beguile')
	#pygame.mouse.set_visible(0)

#Create The Backgound
	bgMangr = BackgroundManager(screen)

#Display The Background
	screen.blit(bgMangr.GetBackground(), (0, 0))
	pygame.display.flip()

#Prepare Game Objects
	clock = pygame.time.Clock()

	global Solids
	from pygame.sprite import Group
	Solids     = Group()

#put in the two platforms to walk on

	level  = GameLevel(bgMangr)

#Main Loop
	oldfps = 0
	while 1:
		clock.tick(40)
		newfps = int(clock.get_fps())
		if newfps is not oldfps:
			print "fps: ", clock.get_fps()
			oldfps = newfps

	#Handle Input Events
		oldEvents = []
		remainingEvents = pygame.event.get()
		for event in remainingEvents:
			oldEvents.append( remainingEvents.pop(0) )
			upKeys = filter( allKUPs, remainingEvents )
			if event.type == QUIT:
				return
			elif event.type == KEYDOWN and event.key == K_ESCAPE:
				return
			elif event.type == KEYDOWN or event.type == KEYUP:
				#if len(upKeys) > 0:
					#print upKeys 
				level.SignalKey( event, upKeys )

	#Draw Everything

		level.DoGraphics( screen, pygame.display )


#Game Over
	pygame.quit()


#this calls the 'main' function when this script is executed
if __name__ == '__main__': main()
