


##these render groups are derived from the normal Group
##class, they just add methods. the Updates and Clear versions
##of drawing are more complex groups. They keep track of sprites
##that have been drawn but are removed, and also keep track of
##drawing info from each sprite, by storing it in the dictionary
##values used to store the sprite. very sneaky, but a great
##example for you if you ever need your own group class that
##maintains its own data along with each sprite it holds.


from pygame.sprite import RenderUpdates

class ScrollSpriteGroup(RenderUpdates):
	"""a sprite group that can draw and clear with update rectangles
	   The RenderUpdates is derived from the RenderClear group
	   and keeps track of all the areas drawn and cleared. It
	   also smartly handles overlapping areas between where a
	   sprite was drawn and cleared when generating the update
	   rectangles."""
	def __init__( self, bgMangr ):
		RenderUpdates.__init__(self)
		self.bgMangr = bgMangr

	def add_internal( self, sprite ):
		RenderUpdates.add_internal( self, sprite )
		center = self.bgMangr.GetDisplayCenter(sprite.rect)
		sprite.displayRect.center = center
		
	def update(self, *args):
		"""update(...)
		   call update for all member sprites

		   calls the update method for all sprites in the group.
		   passes all arguments are to the Sprite update function."""
		if args:
			for s in self.spritedict.keys():
				s.update(args)
				center = self.bgMangr.GetDisplayCenter(s.rect)
				s.displayRect.center = center
				bounds = self.bgMangr.OutOfBounds(s.rect)
				if bounds:
					s.NotifyOutOfBounds( bounds )
				if self.bgMangr.dirty:
					s.NotifyDirtyScreen( self.bgMangr )
		else:
			for s in self.spritedict.keys():
				s.update()
				center = self.bgMangr.GetDisplayCenter(s.rect)
				s.displayRect.center = center
				bounds = self.bgMangr.OutOfBounds(s.rect)
				if bounds:
					s.NotifyOutOfBounds( bounds )
				if self.bgMangr.dirty:
					s.NotifyDirtyScreen( self.bgMangr )


	def clear( self, drawToSurface ):
		if self.bgMangr.dirty:
			#there's no point in clearing things because we're
			#about to wipe the whole surface in update()
			return
		RenderUpdates.clear( self, drawToSurface, 
		                     self.bgMangr.GetBgSurface )
		

	def draw(self, surface):
		"""draw(surface)
		   draw all sprites onto the surface

		   Draws all the sprites onto the given surface. It
		   returns a list of rectangles, which should be passed
		   to pygame.display.update()"""
		if self.bgMangr.dirty:
			self.bgMangr.BlitSelf( surface )
		spritedict = self.spritedict
		surface_blit = surface.blit
		dirty = self.lostsprites
		self.lostsprites = []
		dirty_append = dirty.append
		for s, r in spritedict.items():
			newrect = surface_blit(s.image, s.displayRect)
			if r is 0:
				dirty_append(newrect)
			else:
				if newrect.colliderect(r):
					dirty_append(newrect.union(r))
				else:
					dirty_append(newrect)
					dirty_append(r)
			spritedict[s] = newrect
		if self.bgMangr.dirty:
			self.bgMangr.dirty = 0
			return [self.bgMangr.rect]
		else:
			return dirty



class IdlerSpriteGroup(ScrollSpriteGroup):
	def clear(self, drawToSurface ): pass
	def draw(self): return []
	def update(self):
		if self.bgMangr.dirty:
			for s in self.spritedict.keys():
				s.NotifyDirtyScreen( self.bgMangr )

